@@ -14,12 +14,12 @@ function (object, process = c("Longitudinal", "longitudinal", "Event", "event"),
         fits <- fitted(object, process = "Longitudinal", type = type)
         res <- y - fits
         if (standardized && type %in% c("Subject", "subject"))
-            res <- res / object$coefficients$sigma
+            res <- res / object$postMeans$sigma
         res
     } else {
-        ni <- tapply(object$id, object$id, length)
+        ni <- tapply(object$y$id, object$y$id, length)
         events <- rep(object$y$event, ni)
-        events <- ave(events, object$id, FUN = function (x) c(rep(0, length(x)-1), x[1]))                                            
+        events <- ave(events, object$y$id, FUN = function (x) c(rep(0, length(x)-1), x[1]))                                            
         fits <- fitted(object, process = "Event", 
                        nullY = type %in% c("nullMartingale", "nullmartingale"))
         events - fits
