@@ -1,13 +1,14 @@
 slice.shape <-
-function (logPost, current.shape, step, iters = 6L) {
-    logy <- logPost(current.shape)[[1L]] - rexp(1L)
+function (logPost, current.shapes, step, which, iters = 6L) {
+    current.shape <- current.shapes[which]
+    logy <- logPost(current.shape, which)[[1L]] - rexp(1L)
     tt <- runif(1L, 0, step)
     L <- current.shape - tt
     R <- current.shape + step - tt
-    while (L > 0 && logPost(L)[[1L]] > logy) {
+    while (L > 0 && logPost(L, which)[[1L]] > logy) {
         L <- L - step
     }
-    while (logPost(R)[[1L]] > logy) {
+    while (logPost(R, which)[[1L]] > logy) {
         R <- R + step
     }
     L <- max(0, L)
@@ -15,7 +16,7 @@ function (logPost, current.shape, step, iters = 6L) {
     repeat {
         count <- count + 1L
         new.shape <- runif(1L, L, R)
-        new.logPost <- logPost(new.shape)
+        new.logPost <- logPost(new.shape, which)
         if (new.logPost[[1L]] >= logy || count > iters)
             break
         if (new.shape < current.shape) {
